#ifndef TRACKERGREP_GREPWINDOW_H
#define TRACKERGREP_GREPWINDOW_H

// ***************************************************************************
// System Includes
// ***************************************************************************

#include <be/interface/Button.h>
#include <be/interface/CheckBox.h>
#include <be/interface/MenuField.h>
#include <be/interface/MenuItem.h>
#include <be/interface/OutlineListView.h>
#include <be/interface/TextControl.h>
#include <be/interface/Window.h>

// ***************************************************************************
// Project Includes
// ***************************************************************************

// Standard type and constant definitions.
#include "StyleGuideDefs.h"

// ***************************************************************************
// External Name Declarations
// ***************************************************************************

class Grepper;
class History;

// ***************************************************************************
// Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Type Definitions
// ***************************************************************************

// ***************************************************************************
// Class Definition
// ***************************************************************************

/** 
 * The GrepWindow class is Tracker Grep's main (and only) window.
 * Each window has its own instance of the Grepper class, which
 * allows the user to run multiple copies of Tracker Grep at the
 * same time.
 */
class GrepWindow : public BWindow
{
public:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

	/** Constructor. */
	GrepWindow(entry_ref directory, BMessage* message);

	/** Automatically generated copy constructor. */
	// GrepWindow(const GrepWindow& old);

	/** Destructor. */
	virtual ~GrepWindow(void);

	/** Automatically generated assignment operator. */
	// GrepWindow& operator=(GrepWindow& right);

	/** Responds to notifications that the window has been resized. */
	virtual void FrameResized(float width, float height);
	
	/** Responds to notifications that the window has moved. */
	virtual void FrameMoved(BPoint origin);

	/** Invoked just before a menu belonging to the window is shown. */ 
	virtual void MenusBeginning(void);

	/** Invoked just before a menu belonging to the window is hidden. */ 
	virtual void MenusEnded(void);

	/** Handles the messages that the window receives. */
	virtual void MessageReceived(BMessage* message);
	
protected:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

private:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	/** Our super class. */
	typedef BWindow super;

	// Attribute Definitions =================================================

	/** The search pattern entry field. */
	BTextControl* searchText;
	
	/** The view that will display the results of the search. */
	BOutlineListView* searchResults;

	/** The history pop-up menu. */
	BMenuField* historyMenu;
		
	/** The "Options" menu. */
	BMenuField* optionsMenu;

	/** The "Follow symbolic links" item. */
	BMenuItem* followLinksItem;
	
	/** The "Look in sub-directories" item. */
	BMenuItem* subDirsItem;

	/** The "Do not escape search text" item. */
	BMenuItem* escapeTextItem;

	/** The "Case sensitive" item. */
	BMenuItem* caseSensitiveItem;
	
	/** The "Show contents" check box. */
	BCheckBox* showContents;
	
	/** The "Search/Cancel" button object. */
	BButton* button;

	/** The Grepper object that does all the hard work. */	
	Grepper *grepper;
	
	/** The history of entered items. */
	History* history;
	
	/**
	 * The search pattern. We need to remember it because during the
	 * grepping, the text control's text will be replaced by the name
	 * of the file that's currently being grepped. When the grepping
	 * finishes, we need to restore the search pattern.
	 */
	char* searchPattern;

	/** True when the search is running, false when it is stopped. */
	bool busy;

	// Member Function Definitions ===========================================

	/** Loads the settings for this window. */
	void LoadPrefs(void);
	
	/** Saves the window's settings. */
	void SavePrefs(void);
};

// ***************************************************************************
// Global Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Inline Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Function Declarations
// ***************************************************************************

#endif // TRACKERGREP_GREPWINDOW_H
