// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

#include <be/interface/MenuItem.h>

#include <posix/stdlib.h>
#include <posix/string.h>

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "TrackerGrep.h"
#include "History.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions
// ***************************************************************************

// ===========================================================================
// Constructor
// ===========================================================================

History::History(BLooper* target, int32 limit, const char* filename) 
{
	this->target = target;
	this->limit = limit;
	this->filename = strdup(filename);
	
	history = new BList(limit);
}

// ===========================================================================
// Destructor
// ===========================================================================

History::~History(void) 
{
	free(filename);
	FreeList();
	delete history;
}

// ===========================================================================
// Add
// ===========================================================================

void
History::Add(const char* text) 
{
	FreeList();
	LoadList();

	if (history->CountItems() == limit)
	{
		free(history->RemoveItem((int32) 0));
	}
	
	history->AddItem(strdup(text));

	SaveList();
}

// ===========================================================================
// GetMenu
// ===========================================================================

void
History::FillMenu(BMenu* menu)
{
	FreeList();
	LoadList();

	// Fill up the menu in reverse order.
	for (int32 t = history->CountItems(); t > 0; --t)
	{
		BMessage* message = new BMessage(MSG_HISTORY_ITEM_SELECTED);
		message->AddString("text", (char*) history->ItemAt(t - 1));

		BMenuItem* item = new BMenuItem(
			(char*) history->ItemAt(t - 1),
			message);
			
		menu->AddItem(item);
	}
}

// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ===========================================================================
// FreeList
// ===========================================================================

void
History::FreeList(void)
{
	for (int32 t = history->CountItems(); t > 0; --t)
	{
		free(history->RemoveItem(t - 1));
	}
}

// ===========================================================================
// LoadList
// ===========================================================================

void
History::LoadList(void)
{
	FILE* file = fopen(filename, "r");
	if (file != 0)
	{
		// Each line in the history file is an item.
		char line[1024];
		char* result = fgets(line, 1024, file);
		while (result != 0)
		{
			// Strip off any newlines.
			int32 pos = strlen(line) - 1;
			if (line[pos] == 10)
			{
				line[pos--] = '\0';
			}
			if (line[pos] == 13)
			{
				line[pos--] = '\0';
			}
		
			history->AddItem(strdup(result));
			
			// Can't add more than "limit" items.
			if (history->CountItems() == limit)
			{
				break;
			}
			
			result = fgets(line, 1024, file);
		}

		fclose(file);
	}
}

// ===========================================================================
// SaveList
// ===========================================================================

void
History::SaveList(void)
{
	FILE* file = fopen(filename, "w");
	if (file != 0)
	{
		// Save each item as a line in the history file.		
		for (int32 t = 0; t < history->CountItems(); ++t)
		{
			fputs((char*) history->ItemAt(t), file);
			fputs("\n", file);
		}
		
		fclose(file);
	}
}

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************
