#ifndef TRACKERGREP_HISTORY_H
#define TRACKERGREP_HISTORY_H

// ***************************************************************************
// System Includes
// ***************************************************************************

#include <be/app/Looper.h>
#include <be/interface/Menu.h>
#include <be/support/List.h>

// ***************************************************************************
// Project Includes
// ***************************************************************************

// Standard type and constant definitions.
#include "StyleGuideDefs.h"

// ***************************************************************************
// External Name Declarations
// ***************************************************************************

// ***************************************************************************
// Constant Definitions
// ***************************************************************************

/**
 * The message that is sent when an item from 
 * the history menu is selected.
 */
const int32 MSG_HISTORY_ITEM_SELECTED = 3000;

// ***************************************************************************
// Type Definitions
// ***************************************************************************

// ***************************************************************************
// Class Definition
// ***************************************************************************

/** 
 * The History class keeps track of the search patterns the user
 * entered, and stores them in a file. Because multiple copies of
 * Tracker Grep can be active at the same time, this file also
 * serves as the synchronization mechanism.
 */
class History 
{
public:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

	/** Constructor. */
	History(BLooper* target, int32 limit, const char* filename);

	/** Automatically generated copy constructor. */
	// History(const History& old);

	/** Destructor. */
	virtual ~History(void);

	/** Automatically generated assignment operator. */
	// History& operator=(History& right);

	/** Adds an item to the history. */
	void Add(const char* text);
	
	/** Fills up a BMenu with the items from the history. */
	void FillMenu(BMenu* menu);

protected:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

private:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	/** Pointer to the target BLooper. */
	BLooper* target;

	/** The maximum number of items the history will hold. */
	int32 limit;

	/** The name of the history file. */
	char* filename;

	/** Temporary list of items. */ 
	BList* history;

	// Member Function Definitions ===========================================

	/** Empties the temporary list. */
	void FreeList(void);
	
	/** Loads the history file into the temporary list. */
	void LoadList(void);

	/** Saves the temporary list to the history file. */
	void SaveList(void);
};

// ***************************************************************************
// Global Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Inline Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Function Declarations
// ***************************************************************************

#endif // TRACKERGREP_HISTORY_H
