// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

#include <be/add-ons/tracker/TrackerAddon.h>
#include <be/app/Application.h>
#include <be/interface/Alert.h>
#include <be/storage/Entry.h>

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "TrackerGrep.h"
#include "GrepWindow.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

/** Shows the about window. */
static void ShowAboutWindow(void);

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ===========================================================================
// process_refs
// ===========================================================================

// Paraphrased from the Be Book: The Tracker calls this function when the 
// user invokes the add-on. The current directory is found in "directory",
// "message" is a standard B_REFS_RECEIVED BMessage with the "refs" array 
// containing the entry_refs of the files selected by the user. The third 
// argument is currently unused. 

void 
process_refs(entry_ref directory, BMessage* message, void* reserved) 
{ 
	// Create the window. This starts our main loop.
	GrepWindow* window = new GrepWindow(directory, message);
	
	// Wait until the window thread terminates. If we don't do this,
	// our add-on might crash mysteriously, taking the Tracker with
	// it. See also "Be Newsletter Volume II Issue 10".
	int32 exitValue;
	wait_for_thread(window->Thread(), &exitValue);
} 
   
// ===========================================================================
// main
// ===========================================================================

// An add-on is not required to have a main() function. But if it
// does, the user can run the add-on as a stand-alone application
// as well. We just show a simple about window.

int
main(void) 
{
	BApplication app("application/x-vnd.savvy.trackergrep");
	ShowAboutWindow();
	return 0;
}

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************

// ===========================================================================
// ShowAboutWindow
// ===========================================================================

void
ShowAboutWindow(void)
{
	(new BAlert(0,
		"Tracker Grep -- Use grep from the Tracker\n\n"
		"Version " VERSION_STRING " (" VERSION_DATE_STRING ")\n\n"
		"Created by Matthijs Hollemans\n"
		"mahlzeit@bigfoot.com\n"
		"http://home.concepts.nl/~hollies/\n"
		"http://www.weirdexperiments.com\n\n"
		"Thanks to Peter Hinely, Serge Fantino,\n"
		"and Hideki Naito for their help.\n",
		"Okay",
		0,
		0,
		B_WIDTH_AS_USUAL,
		B_INFO_ALERT))->Go();
}
